//------------------------------------------------------------------------------
// <copyright file="WebDataService.svc.cs" company="Microsoft">
//     Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Data.Entity.Infrastructure;
using System.Data.Services;
using System.Data.Services.Client;
using System.Data.Services.Common;
using System.Linq;
using System.Linq.Expressions;
using System.ServiceModel.Web;
using System.Web;

namespace ChinookWCFDataService
{
    public class ChinooDataService : DataService<ChinookEntities>
    {
        // This method is called only once to initialize service-wide policies.
        public static void InitializeService(DataServiceConfiguration config)
        {
            // TODO: set rules to indicate which entity sets and service operations are visible, updatable, etc.
            // Examples:
            config.UseVerboseErrors = true;
            config.SetEntitySetAccessRule("*", EntitySetRights.All);
            config.SetServiceOperationAccessRule("*", ServiceOperationRights.All);
            config.DataServiceBehavior.MaxProtocolVersion = DataServiceProtocolVersion.V3;
        }


        [WebGet]
        public IQueryable<ArtistAlphabetInfo> ArtistSummary()
        {

            var data = (from oneArtist in CurrentDataSource.Artists
                        group oneArtist by oneArtist.Name.Substring(0, 1) into letterGroup
                        select new { Letter = letterGroup.Key, Count = letterGroup.Count() }).ToList();
            return data.Select(one => new ArtistAlphabetInfo { Letter = one.Letter, Total = one.Count }).AsQueryable();
        }

        [WebInvoke]
        public void UpdateRandomArtist(int artistId)
        {
            var artist = CurrentDataSource.Artists.FirstOrDefault(one => one.ArtistId == artistId);
            if (artist != null)
            {
                artist.Name = artist.Name + "_upd";
                CurrentDataSource.SaveChanges();
            }

        }

        [QueryInterceptor("Artists")]
        public Expression<Func<Artist, bool>> OnQueryArtists()
        {
            if (!HttpContext.Current.Request.IsAuthenticated)
                return artist => false;
            else
            {
                if (!HttpContext.Current.User.Identity.Name.Contains("a"))
                {
                    return artist => artist.Name.Contains("a");
                }
                else
                {
                    return artist => true;
                }
            }
            
        }

        [ChangeInterceptor("Artists")]
        public void OnChangeArtist(Artist artist, UpdateOperations updateOperation)
        {
            if (!HttpContext.Current.Request.IsAuthenticated)
                throw new DataServiceException(401, "Cannot update Artists without logging in");
            if (!HttpContext.Current.User.Identity.Name.Contains("a"))
            {
                throw new DataServiceException(401, "Cannot update Artists without logging in");
            }
        }
    }
}
