﻿using System.Diagnostics;
using System.Net;
using System.Security;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data.Services.Client;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Core;
using Windows.UI.Popups;

namespace ChinookWinApp.Common
{
    public static class WcfDataServicesExtensions
    {

        public static async Task<IEnumerable<TResult>> ExecuteAsync<TResult>(this DataServiceQuery<TResult> query)
        {
            var queryTask = Task.Factory.FromAsync<IEnumerable<TResult>>(query.BeginExecute(null, null), (asResult) =>
            {
                var result = query.EndExecute(asResult).ToList();
                return result;
            });

            return await queryTask;
        }

        public static async Task<ObservableCollection<TResult>> ExecuteAsync<TResult>(this DataServiceContext context, Uri uri, Cookie cookie)
        {
            HttpClientHandler handler = new HttpClientHandler();
            handler.CookieContainer = new CookieContainer();
            string value = "";
            handler.CookieContainer.Add(uri, cookie);
            var client = new HttpClient(handler);
            client.DefaultRequestHeaders.Add("Accept", "application/json;odata=verbose");

            var data = await client.GetAsync(uri);
            var text = await data.Content.ReadAsStringAsync();
            var test = JsonConvert.DeserializeObject<IntermediateResult<TResult>>(text);
            return test.Data;
        }

        public static async Task SaveAsync(this DataServiceContext context, CoreDispatcher dispatcher)
        {
            var saveTask = Task.Factory.FromAsync(context.BeginSaveChanges(SaveChangesOptions.Batch, null, context), (result) =>
                {
                    dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                       {
                           var returnValue = new List<Exception>();

                           if (result.IsCompleted)
                           {
                               var client = (DataServiceContext)result.AsyncState;
                               try
                               {
                                   DataServiceResponse response = client.EndSaveChanges(result);
                                   foreach (ChangeOperationResponse change in response)
                                   {
                                       if (change.Error != null)
                                       {
                                           returnValue.Add(change.Error);
                                       }
                                   }
                                   if (returnValue.Any())
                                   {
                                       var dialog = new MessageDialog("Unknown error");
                                       dialog.ShowAsync();
                                   }
                               }
                               catch (Exception exception)
                               {
                                   if (exception.InnerException != null &&
                                       exception.InnerException.ToString().ToLower().Contains("cannot update"))
                                   {
                                       var dialog = new MessageDialog("You do not have rights to update");
                                       dialog.ShowAsync();
                                   }
                                   else
                                   {
                                       var dialog = new MessageDialog("Unknown error");
                                       dialog.ShowAsync();
                                   }   
                               }
                               
                           }
                       });

                });

            await saveTask;
        }

        public static async Task<Cookie> Login(this DataServiceContext context, string userName, string password)
        {
            HttpClientHandler handler = new HttpClientHandler();
            handler.CookieContainer = new CookieContainer();
            var client = new HttpClient(handler);

            //var uri = new Uri(context.BaseUri.ToString().Replace(@"ChinooDataService.svc/", @"LoginService.svc/Login?userName=" + userName + "&password=" + password));
            var uri = new Uri(context.BaseUri.ToString().Replace(@"ChinooDataService.svc/", @"Authentication_JSON_AppService.axd/Login"));

            string authBody = String.Format(
                "{{ \"userName\": \"{0}\", \"password\": \"{1}\", \"createPersistentCookie\":true}}",
                userName,
                password);

            var responce = await client.PostAsync(uri, new StringContent(authBody, new UTF8Encoding(), "application/json"));
            //var responce = await client.GetAsync(uri);
            var cookies = handler.CookieContainer.GetCookies(uri);
            if (cookies.Count == 0)
            {
                return null;
            }
            return cookies[".ASPXAUTH"];

        }
    }
}
